"""
_summary_

:return: _description_
:rtype: _type_
"""

import os
import random
import time

import requests
from tqdm.notebook import tqdm, trange


def download_urls(urls, path, get_filename_from_url=True):
    """
    Function to download list of files with a progress bar.


    :param urls: Lista ou tuple, a depender da função #
    :param path: Local onde os arquivos serão inseridos
    :param get_filename_from_url: Se VERDADEIRO, a função irá nomear o arquivo conforme o link das lista das URLs.
    Se FALSO, a função irá nomear o arquivo conforme string definida no tuple das URLs
    """

    # Reset Interactions
    i = 0
    n_urls = len(urls)

    for n_url in trange(n_urls, desc='Total'):
        # Download path and file name
        if get_filename_from_url:
            url = urls[i]
            filename = urls[i].rsplit('/', 1)[1]
        else:
            url = urls[i][0]
            filename = urls[i][1]

        # File size
        r = requests.get(url, stream=True)
        chunk_size = 1024 * 1024
        total_size = int(r.headers['content-length'])

        # Download the file from 'url' and save it locally under 'filename'
        with open(os.path.join(path, filename), 'wb') as f:
            for data in tqdm(
                iterable=r.iter_content(chunk_size=chunk_size),
                total=int(total_size / chunk_size),
                unit='MB',
                desc=f'{i + 1}/{n_urls}',
            ):

                f.write(data)
                time.sleep(0.1)

        # Interactions
        i += 1

        # Definir um intervalo de tempo
        time.sleep(random.randint(3, 8))


def download_file_upper(url):
    """
    _summary_

    :param url: _description_
    :type url: _type_
    """
    local_filename = url.split('/')[-1]
    r = requests.get(url)
    f = open(local_filename, 'wb')
    for chunk in r.iter_content(chunk_size=512 * 1024):
        time.sleep(0.03)

        if chunk:  # filter out keep-alive new chunks
            f.write(chunk)
    f.close()
    return


def download_file(url):
    """
    _summary_

    :param url: _description_
    :type url: _type_
    :return: _description_
    :rtype: _type_
    NOTE the stream=True parameter below
    """
    local_filename = url.split('/')[-1]
    with requests.get(url, timeout=60, stream=True) as r:
        r.raise_for_status()
        with open(local_filename, 'wb') as f:
            for chunk in r.iter_content(chunk_size=8192):
                # If you have chunk encoded response uncomment if
                # and set chunk_size parameter to None.
                # if chunk:
                f.write(chunk)
    return local_filename


if __name__ == '__main__':
    urls = [
        'https://sage.saude.gov.br/dados/sisagua/cadastro_pontos_captacao.zip',
        'https://sage.saude.gov.br/dados/sisagua/cadastro_tratamento_de_agua.zip',
        'https://sage.saude.gov.br/dados/sisagua/cadastro_populacao_abastecida.zip',
    ]

    path = os.path.abspath(os.path.join(os.getcwd(), '..', '..', '..', 'data'))
    print(path)

    download_urls(urls, path)
    # DownloadFile(urls[1])
    # download_file(urls[1])



"""
_summary_

:return: _description_
:rtype: _type_
"""

import random

import requests
from pycep_correios import WebService, get_address_from_cep

# # Set Version
# VERSION = (1, 0, 15) #VERSION = (1, 0, 7, 'dev0')
# __version__ = '.'.join(map(str, VERSION))


# Functions
def get_list_ceps_bairros(estado='sp', municipio='piracicaba', loops=20):
    """
    Função que auxilia na definição de endereços aleatórios em uma cidade.

    :param estado: sigla da unidade da federação (default: "sp")
    :param municipio: nome do município (default: "piracicaba")
    :param loop: quanto maior o número de loops, maior a lista  (default: 20)
    :return: Output consiste em duas listas (list_ceps, list_bairros)
    """
    termos = [
        'rua',
        'avenida',
        'praça',
        'bosque',  # Padrão
        'janeiro',
        'fevereiro',
        'março',
        'abril',
        'maio',
        'junho',
        'julho',
        'agosto',
        'setembro',
        'outubro',
        'novembro',
        'dezembro',  # Meses
        'marechal',
        'sargento',
        'duque',
        'soldado',
        'capitão',
        'major',  # Patentes
        'dom',
        'senhor',
        'senhora',  # Pronomes de Tratamento
        'almeida',
        'caetano',
        'pedro',
        'barbosa',
        'rui',
        'pinto',
        'joão',
        'são',  # Nomes
    ]
    i = 1
    list_ceps = []
    list_bairros = []
    while i <= loops:
        url = f'https://viacep.com.br/ws/{estado}/{municipio}/{random.choice(termos)}/json/'
        resp = requests.get(url)
        json = resp.json()
        for data in json:
            list_ceps.append(data['cep'])
            list_bairros.append(data['bairro'])
        i += 1

    # CEPs
    list_ceps = list(set(list_ceps))
    list_ceps.sort()

    # Bairros
    list_bairros = list(set(list_bairros))
    list_bairros.sort()

    print(
        f'Município: {municipio.title()}-{estado.upper()}\nCEPs distintos: {len(list_ceps)}\nBairros distintos: {len(list_bairros)}'
    )
    return list_ceps, list_bairros


def get_random_complete_address(cep):
    """
    Pega um endereço a parte de um CEP

    :param cep: Definição do CEP que se deseja obter o endereço (Exemplo: '13419-398')
    :return: String com o endereço completo.

    """
    # Endereço
    end = get_address_from_cep(cep, webservice=WebService.VIACEP)

    # Adiciona um número aleatório
    end['numero'] = f'{random.randrange(1, 999)}'

    # Cria uma chave no dict com Endereço Completo
    end[
        'endereco_completo'
    ] = f'{end["logradouro"]}, {end["numero"]} - {end["bairro"]} - {end["cidade"]}, {end["uf"]} - CEP: {end["cep"]} '
    print(f'Endereço Aleatório:\n{end["endereco_completo"]}')
    return {
        'logradouro': end['logradouro'],
        'numero': end['numero'],
        'bairro': end['bairro'],
        'cidade': end['cidade'],
        'uf': end['uf'],
        'cep': end['cep'],
        'endereco_completo': end['endereco_completo'],
    }


if __name__ == '__main__':
    # Para testes apenas
    list_ceps, list_bairros = get_list_ceps_bairros(
        estado='sp', municipio='piracicaba'
    )
    dict_endereco = get_random_complete_address(random.choice(list_ceps))
    print(dict_endereco)
